@@ -36,33 +36,25 @@
 # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 # OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from asyncio import (
-    CancelledError, get_event_loop, Queue, Event, Lock, Semaphore, sleep
-)
 from collections import deque
 from contextlib import suppress
 from functools import partial
 import logging
 import sys
 
+from aiorpcx.asyncio import (
+    CancelledError, get_event_loop, Queue, Event, Lock, Semaphore, sleep, current_task
+)
 from aiorpcx.util import normalize_corofunc, check_task
 
 
 __all__ = (
-    'Queue', 'Event', 'Lock', 'Semaphore', 'sleep', 'CancelledError',
     'run_in_thread', 'spawn', 'spawn_sync', 'TaskGroup',
     'TaskTimeout', 'TimeoutCancellationError', 'UncaughtTimeoutError',
     'timeout_after', 'timeout_at', 'ignore_after', 'ignore_at',
 )
 
 
-if sys.version_info >= (3, 7):
-    from asyncio import current_task
-else:
-    from asyncio import Task
-    current_task = Task.current_task
-
-
 async def run_in_thread(func, *args):
     '''Run a function in a separate thread, and await its completion.'''
     return await get_event_loop().run_in_executor(None, func, *args)
@@ -212,8 +204,6 @@ class TaskGroup(object):
         task_list = list(self._pending)
         for task in task_list:
             task.cancel()
-        # Give event loop chance to process cancellations
-        await sleep(0)
         for task in task_list:
             with suppress(CancelledError):
                 await task
