@@ -27,7 +27,7 @@
 
 __all__ = ('JSONRPC', 'JSONRPCv1', 'JSONRPCv2', 'JSONRPCLoose',
            'JSONRPCAutoDetect', 'Request', 'Notification', 'Batch',
-           'RPCError', 'ProtocolError',
+           'RPCError', 'FinalRPCError', 'ProtocolError',
            'JSONRPCConnection', 'handler_invocation')
 
 import itertools
@@ -140,6 +140,10 @@ class RPCError(CodeMessageError):
     pass
 
 
+class FinalRPCError(RPCError):
+    pass
+
+
 class ProtocolError(CodeMessageError):
 
     def __init__(self, code, message):
